@@ -1,19 +1,13 @@
-#############################################################################
-#   $Author: markus $
-#     $Date: 2009-09-07 19:54:33 +0200 (Mon, 07 Sep 2009) $
-# $Revision: 1837 $
-#############################################################################
-
 package LaTeX::Table;
 
 use strict;
 use warnings;
 
-use Moose::Policy 'Moose::Policy::FollowPBP';
 use Moose;
 use Moose::Util::TypeConstraints;
+use MooseX::FollowPBP;
 
-use version; our $VERSION = qv('1.0.0');
+use version; our $VERSION = qv('1.0.6');
 
 use LaTeX::Table::Types::Std;
 use LaTeX::Table::Types::Xtab;
@@ -34,7 +28,9 @@ use Module::Pluggable
 
 # Str
 for my $attr (
-    qw(label maincaption shortcaption caption caption_top coldef custom_template width maxwidth width_environment custom_tabular_environment position tabletail star)
+    qw(label maincaption shortcaption caption caption_top coldef
+    custom_template width maxwidth width_environment
+    custom_tabular_environment position tabletail star)
     )
 {
     has $attr => ( is => 'rw', isa => 'Str', default => 0 );
@@ -42,6 +38,7 @@ for my $attr (
 
 has 'filename'  => ( is => 'rw', isa => 'Str', default => 'latextable.tex' );
 has 'foottable' => ( is => 'rw', isa => 'Str', default => q{} );
+has 'eor'       => ( is => 'rw', isa => 'Str', default => q{\\\\} );
 has 'environment'  => ( is => 'rw', isa => 'Str', default => 1 );
 has 'theme'        => ( is => 'rw', isa => 'Str', default => 'Meyrin' );
 has 'continuedmsg' => ( is => 'rw', isa => 'Str', default => '(continued)' );
@@ -112,9 +109,6 @@ __PACKAGE__->meta->make_immutable;
 sub generate_string {
     my ( $self, @args ) = @_;
 
-    # are the user provided options ok?
-    $self->_check_options();
-
     # analyze the data
     $self->_calc_data_summary( $self->get_data );
 
@@ -122,9 +116,9 @@ sub generate_string {
         = 'LaTeX::Table::Types::'
         . uc( substr $self->get_type, 0, 1 )
         . substr $self->get_type, 1;
-    $self->set__type_obj( $type_obj_name->new( _table_obj => $self ) );
+    $self->_set_type_obj( $type_obj_name->new( _table_obj => $self ) );
 
-    return $self->get__type_obj->generate_latex_code();
+    return $self->_get_type_obj->generate_latex_code();
 }
 
 sub generate {
@@ -280,7 +274,7 @@ ROW:
         }
         push @summary, $type_of_this_col;
     }
-    $self->set__data_summary( \@summary );
+    $self->_set_data_summary( \@summary );
     return;
 }
 
@@ -302,7 +296,7 @@ sub _get_matrix_latex_code {
 
     my @code
         = $is_header
-        ? ( $self->_get_hline_code( $self->get__RULE_TOP_ID ) )
+        ? ( $self->_get_hline_code( $self->_get_RULE_TOP_ID ) )
         : ();
 ROW:
     for my $row ( @{$data_ref} ) {
@@ -312,7 +306,7 @@ ROW:
         # empty rows produce a horizontal line
         if ( !@cols ) {
             push @code,
-                $self->_get_hline_code( $self->get__RULE_INNER_ID, 1 );
+                $self->_get_hline_code( $self->_get_RULE_INNER_ID, 1 );
             next ROW;
         }
 
@@ -351,16 +345,16 @@ ROW:
 
         # do we have to draw a horizontal line?
         if ( $i == scalar @{$data_ref} ) {
-            push @code, $self->_get_hline_code( $self->get__RULE_BOTTOM_ID );
+            push @code, $self->_get_hline_code( $self->_get_RULE_BOTTOM_ID );
         }
         else {
-            push @code, $self->_get_hline_code( $self->get__RULE_INNER_ID );
+            push @code, $self->_get_hline_code( $self->_get_RULE_INNER_ID );
         }
     }
 
     # without header, just draw the topline, not this midline
     if ( $is_header && $i ) {
-        push @code, $self->_get_hline_code( $self->get__RULE_MID_ID );
+        push @code, $self->_get_hline_code( $self->_get_RULE_MID_ID );
     }
 
     return $self->_align_code( \@code );
@@ -390,7 +384,7 @@ ROW:
         for my $i ( 0 .. scalar( @{$row} ) - 1 ) {
             $row->[$i] = sprintf '%-*s', $max{$i}, $row->[$i];
         }
-        $code .= join( ' & ', @{$row} ) . " \\\\\n";
+        $code .= join( ' & ', @{$row} ) . q{ } .  $self->get_eor . "\n";
     }
     return $code;
 }
@@ -405,7 +399,7 @@ sub _get_hline_code {
     {
         $line = $theme->{RULES_CMD}->[$id];
     }
-    if ( $id == $self->get__RULE_BOTTOM_ID ) {
+    if ( $id == $self->_get_RULE_BOTTOM_ID ) {
         $id = 0;
     }
 
@@ -477,7 +471,7 @@ sub _get_row_array {
                     )
                 );
                 if ( !defined $col_def->{cols} ) {
-                    my @summary = @{ $self->get__data_summary() };
+                    my @summary = @{ $self->_get_data_summary() };
                     $col_def->{cols} = 1;
                     $col_def->{align}
                         = $self->get_coldef_strategy->{ $summary[$col_id]
@@ -541,7 +535,7 @@ sub _get_mc_value {
 
 sub _get_mc_def {
     my ( $self, $value ) = @_;
-    return $value =~ m{ \A (.*)\:(\d)([clr]) \z }xms
+    return $value =~ m{ \A (.*)\:(\d+)([clr]) \s* \z }xms
         ? {
         value => $1,
         cols  => $2,
@@ -574,12 +568,10 @@ sub _add_font_color {
 
 sub _get_coldef_type_col_suffix {
     my ($self) = @_;
-    if (   $self->get_width_environment eq 'tabularx'
-        || $self->get_type eq 'ctable' )
-    {
+    if ( $self->get_width_environment eq 'tabularx' ) {
         return '_COL_X';
     }
-    if ( $self->get_width_environment eq 'tabulary' ) {
+    elsif ( $self->get_width_environment eq 'tabulary' ) {
         return '_COL_Y';
     }
     return '_COL';
@@ -587,7 +579,7 @@ sub _get_coldef_type_col_suffix {
 
 sub _get_coldef_code {
     my ( $self, $data ) = @_;
-    my @cols   = @{ $self->get__data_summary() };
+    my @cols   = @{ $self->_get_data_summary() };
     my $vlines = $self->get_theme_settings->{'VERTICAL_RULES'};
 
     my $v0 = q{|} x $vlines->[0];
@@ -634,8 +626,7 @@ sub _get_coldef_code {
         $i++;
         if (   $i == 1
             && $self->get_width
-            && !$self->get_width_environment
-            && $self->get_type ne 'ctable' )
+            && !$self->get_width_environment )
         {
             $table_def .= '@{\extracolsep{\fill}}';
         }
@@ -654,67 +645,6 @@ sub get_theme_settings {
     return;
 }
 
-sub _check_options {
-    my ($self) = @_;
-
-    # default floating enviromnent is table
-    if ( $self->get_environment eq '1' ) {
-        $self->set_environment('table');
-    }
-
-    if ( $self->get_type eq 'xtab' || $self->get_type eq 'longtable' ) {
-        if ( !$self->get_environment ) {
-            $self->_invalid_option_usage( 'environment',
-                'xtab/longtable requires an environment' );
-        }
-        if ( $self->get_position ) {
-            $self->_invalid_option_usage( 'position',
-                'xtab/longtable does not support position' );
-        }
-    }
-
-    # check center, right, left options
-    my $cnt_true_alignments = 0;
-    for my $align ( $self->get_center, $self->get_right, $self->get_left ) {
-        if ($align) {
-            $cnt_true_alignments++;
-        }
-    }
-    if ( $cnt_true_alignments > 1 ) {
-        $self->_invalid_option_usage( 'center, left, right',
-            'only one allowed.' );
-    }
-    if ( $self->has_center || $self->has_right || $self->has_left ) {
-        $self->set__default_align(0);
-    }
-    else {
-        $self->set__default_align(1);
-    }
-
-    if ( $self->get_maincaption && $self->get_shortcaption ) {
-        $self->_invalid_option_usage( 'maincaption, shortcaption',
-            'only one allowed.' );
-    }
-
-    # handle default values by ourselves
-    if ( $self->get_width_environment eq 'tabular*' ) {
-        $self->set_width_environment(0);
-    }
-    if ( !$self->get_width ) {
-        if (   $self->get_width_environment eq 'tabularx'
-            && $self->get_type ne 'longtable' )
-        {
-            $self->_invalid_option_usage( 'width_environment',
-                'Is tabularx and width is unset' );
-        }
-        elsif ( $self->get_width_environment eq 'tabulary' ) {
-            $self->_invalid_option_usage( 'width_environment',
-                'Is tabulary and width is unset' );
-        }
-    }
-    return;
-}
-
 no Moose::Util::TypeConstraints;
 no Moose;
 1;    # Magic true value required at end of module
@@ -726,7 +656,7 @@ LaTeX::Table - Perl extension for the automatic generation of LaTeX tables.
 
 =head1 VERSION
 
-This document describes LaTeX::Table version 1.0.0
+This document describes LaTeX::Table version 1.0.6
 
 =head1 SYNOPSIS
 
@@ -753,7 +683,7 @@ This document describes LaTeX::Table version 1.0.0
         maincaption => 'Price List',
         caption     => 'Try our special offer today!',
         label       => 'table:prices',
-        position    => 'htb',
+        position    => 'tbp',
         header      => $header,
         data        => $data,
   	}
@@ -1072,9 +1002,9 @@ Default is 0. Requires C<environment>.
 
 =item C<position>
 
-The position of the environment, e.g. 'htb'. Only generated if get_position()
+The position of the environment, e.g. 'tbp'. Only generated if get_position()
 returns a true value. Default 0. Requires C<environment> and tables of C<type>
-I<std>.
+I<std> or I<ctable>.
 
 =item C<sideways>
 
@@ -1224,8 +1154,9 @@ This will produce following LaTeX code:
   \begin{tabular*}{0.75\textwidth}{l@{\extracolsep{\fill} ... }
 
 For tables of C<type> I<std>, it is also possible to use the C<tabularx> and
-C<tabulary> LaTeX packages (see C<width_environment> below). The tables of type I<ctable>
-automatically use the C<tabularx> package.
+C<tabulary> LaTeX packages (see C<width_environment> below). The tables of
+type I<ctable> automatically use the C<tabularx> package. See also
+C<width_environment> for how to use this feature with I<longtable>. 
 
 =item C<width_environment>
 
@@ -1268,6 +1199,15 @@ Default is 0 (see C<width>).
 
 Only supported by tables of type I<ctable>. 
 
+=item C<eor>
+
+String specifying the end of a row. Default is '\\'.
+  
+  $table->set_eor("\\\\[1em]");
+
+Callback functions (see below) can be used to manually set the eor after the last
+column. This is useful when some rows require different eor strings. 
+
 =item C<callback>
 
 If get_callback() returns a true value and the return value is a code reference,
@@ -1484,6 +1424,8 @@ L<Data::Table>, L<LaTeX::Encode>
 
 =over
 
+=item ANDREWF, ANSGAR and REHSACK for some great patches and suggestions.
+
 =item David Carlisle for the C<colortbl>, C<longtable>, C<ltxtable>,
 C<tabularx> and C<tabulary> LaTeX packages.
 
@@ -1492,9 +1434,6 @@ C<tabularx> and C<tabulary> LaTeX packages.
 =item Simon Fear for the C<booktabs> LaTeX package. The L<"SYNOPSIS"> table is
 the example in his documentation.
 
-=item Andrew Ford (ANDREWF) for many great suggestions. He also wrote
-L<LaTeX::Driver> and L<LaTeX::Encode> which are used by I<csv2pdf>.
-
 =item Lapo Filippo Mori for the excellent tutorial I<Tables in LaTeX2e:
 Packages and Methods>.
 
@@ -1502,13 +1441,9 @@ Packages and Methods>.
 
 =back
 
-=head1 AUTHOR
-
-Markus Riester  C<< <mriester@gmx.de> >>
-
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2006-2009, Markus Riester C<< <mriester@gmx.de> >>. 
+Copyright (c) 2006-2010 C<< <limaone@cpan.org> >> 
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself. See L<perlartistic>.
